"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mochaGlobalSetup = exports.configList = void 0;
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
const common_utils_1 = require("@fluidframework/common-utils");
const testApi_1 = require("./testApi");
const packageVersion_1 = require("./packageVersion");
const compatOptions_1 = require("./compatOptions");
// N and N - 1
const defaultVersions = [0, -1];
// we are currently supporting 0.45 long-term
const LTSVersions = ["^0.45.0"];
function genConfig(compatVersion) {
    if (compatVersion === 0) {
        return [
            {
                // include the base version if it is not the same as the package version and it is not the test build
                name: `Non-Compat${compatOptions_1.baseVersion !== packageVersion_1.pkgVersion ? ` v${compatOptions_1.baseVersion}` : ""}`,
                kind: compatOptions_1.CompatKind.None,
                compatVersion: 0,
            },
        ];
    }
    const allOld = {
        loader: compatVersion,
        driver: compatVersion,
        containerRuntime: compatVersion,
        dataRuntime: compatVersion,
    };
    const compatVersionStr = typeof compatVersion === "string" ? compatVersion : `N${compatVersion}`;
    return [
        {
            name: `compat ${compatVersionStr} - old loader`,
            kind: compatOptions_1.CompatKind.Loader,
            compatVersion,
            loader: compatVersion,
        },
        Object.assign(Object.assign({ name: `compat ${compatVersionStr} - new loader`, kind: compatOptions_1.CompatKind.NewLoader, compatVersion }, allOld), { loader: undefined }),
        {
            name: `compat ${compatVersionStr} - old driver`,
            kind: compatOptions_1.CompatKind.Driver,
            compatVersion,
            driver: compatVersion,
        },
        Object.assign(Object.assign({ name: `compat ${compatVersionStr} - new driver`, kind: compatOptions_1.CompatKind.NewDriver, compatVersion }, allOld), { driver: undefined }),
        {
            name: `compat ${compatVersionStr} - old container runtime`,
            kind: compatOptions_1.CompatKind.ContainerRuntime,
            compatVersion,
            containerRuntime: compatVersion,
        },
        Object.assign(Object.assign({ name: `compat ${compatVersionStr} - new container runtime`, kind: compatOptions_1.CompatKind.NewContainerRuntime, compatVersion }, allOld), { containerRuntime: undefined }),
        {
            name: `compat ${compatVersionStr} - old data runtime`,
            kind: compatOptions_1.CompatKind.DataRuntime,
            compatVersion,
            dataRuntime: compatVersion,
        },
        Object.assign(Object.assign({ name: `compat ${compatVersionStr} - new data runtime`, kind: compatOptions_1.CompatKind.NewDataRuntime, compatVersion }, allOld), { dataRuntime: undefined }),
    ];
}
const genLTSConfig = (compatVersion) => {
    return [
        {
            name: `compat LTS ${compatVersion} - old loader`,
            kind: compatOptions_1.CompatKind.Loader,
            compatVersion,
            loader: compatVersion,
        },
        {
            name: `compat LTS ${compatVersion} - old loader + old driver`,
            kind: compatOptions_1.CompatKind.LoaderDriver,
            compatVersion,
            driver: compatVersion,
            loader: compatVersion,
        },
    ];
};
exports.configList = new common_utils_1.Lazy(() => {
    // set it in the env for parallel workers
    if (compatOptions_1.compatKind) {
        process.env.fluid__test__compatKind = JSON.stringify(compatOptions_1.compatKind);
    }
    if (compatOptions_1.compatVersions) {
        process.env.fluid__test__compatVersion = JSON.stringify(compatOptions_1.compatVersions);
    }
    process.env.fluid__test__driver = compatOptions_1.driver;
    process.env.fluid__test__r11sEndpointName = compatOptions_1.r11sEndpointName;
    process.env.fluid__test__tenantIndex = compatOptions_1.tenantIndex.toString();
    process.env.fluid__test__baseVersion = compatOptions_1.baseVersion;
    let _configList = [];
    if (!compatOptions_1.compatVersions || compatOptions_1.compatVersions.length === 0) {
        defaultVersions.forEach((value) => {
            _configList.push(...genConfig(value));
        });
        LTSVersions.forEach((value) => {
            _configList.push(...genLTSConfig(value));
        });
    }
    else {
        compatOptions_1.compatVersions.forEach((value) => {
            if (value === "LTS") {
                LTSVersions.forEach((lts) => {
                    _configList.push(...genLTSConfig(lts));
                });
            }
            else {
                const num = parseInt(value, 10);
                if (num.toString() === value) {
                    _configList.push(...genConfig(num));
                }
                else {
                    _configList.push(...genConfig(value));
                }
            }
        });
    }
    if (compatOptions_1.compatKind !== undefined) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        _configList = _configList.filter((value) => compatOptions_1.compatKind.includes(value.kind));
    }
    return _configList;
});
/*
 * Mocha start up to ensure legacy versions are installed
 */
async function mochaGlobalSetup() {
    const versions = new Set(exports.configList.value.map((value) => value.compatVersion));
    if (versions.size === 0) {
        return;
    }
    // Make sure we wait for all before returning, even if one of them has error.
    const installP = Array.from(versions.values()).map(async (value) => (0, testApi_1.ensurePackageInstalled)(compatOptions_1.baseVersion, value, compatOptions_1.reinstall));
    let error;
    for (const p of installP) {
        try {
            await p;
        }
        catch (e) {
            error = e;
        }
    }
    if (error) {
        throw error;
    }
}
exports.mochaGlobalSetup = mochaGlobalSetup;
//# sourceMappingURL=compatConfig.js.map